// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package router

import (
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/controller"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/core"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/errno"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/middleware"
	"github.com/gin-contrib/cors"
	"github.com/gin-gonic/gin"
)

func Routes(engine *gin.Engine, c controller.AppController) *gin.Engine {
	// Middlewares
	// Cors
	corsCfg := cors.DefaultConfig()
	corsCfg.AllowAllOrigins = true
	corsCfg.AllowHeaders = append(corsCfg.AllowHeaders, "Authorization", "Accept", "X-CSRF-Token")
	engine.Use(cors.New(corsCfg))

	// Use my request id middleware
	// TODO: I might use the community version later
	engine.Use(middleware.RequestID())

	// Route for the 404 error
	engine.NoRoute(func(ctx *gin.Context) {
		core.WriteResponse(ctx, errno.PageNotFoundErr, nil)
	})

	v1 := engine.Group("/v1")
	{
		userV1 := v1.Group("/user")
		{
			userV1.POST("/create", func(ctx *gin.Context) { c.User.Create(ctx) })
		}
	}

	return engine
}
